/*============================================================*

	-ParseFile.cpp-	

All content  Copyright 2001 Nicholas Cooper.
By Smoogle - Saturday, 23 March 2002 - Initial revision
*============================================================*/

#include <string.h>
#include "ParseFile.h"

ParseFile::ParseFile(const char* filename)
{
	
	strcpy(mFilename, filename);
	mFile = 0;
	FILE* file = fopen(mFilename, "rb");

	if(!file)
	{
		return;
	}

	fseek(file, 0, SEEK_END);
	mFilelen = ftell(file);
	fseek(file, 0, SEEK_SET);

	mFile = new char[mFilelen+1];
	mPosition = mFile;
	mToken[0] = mWorkingBuffer[0] = 0;

	int pos = 0;
	while(!feof(file))
	{
		char c = fgetc(file);

		switch(c)
		{
		case '\r':
			continue;
//		case '\t':
//			c = ' ';
		default: break;
		}

		mFile[pos++] = c;
	}
	mFile[pos-1] = 0;

	fclose(file);
}

ParseFile::~ParseFile()
{
	if(mFile)
		delete[] mFile;
}

bool ParseFile::Comment()
{
	if(mFile && *mPosition == ';') 
	{
		SkipLine();
/*#else
	if(mFile && *mPosition == '/')
	{
		// check the next character, for either // or 
		switch(*(mPosition+1))
		{
		case '*':
			
			// skip everything until we reach EOF or 
			while(*mPosition && !(*mPosition == '*' && *(mPosition+1) == '/'))
				mPosition++;
			
			// if we didn't hit EOF, move two along to skip the 
			if(*mPosition)
			{
				mPosition += 2;
			}
			break;
			
			// if we have a //
		case '/':
			// skip the line
			SkipLine();
			break;
			
		default:
			// nothing
			return false;
		}
#endif		*/
		// we had a comment, return true
		return true;
	}
	// noting
	return false;
}


// skip a line
void ParseFile::SkipLine()
{
	// ignore everything until we hit a newline
	while(*mPosition && *mPosition != '\n')
	{
		mPosition++;
	}
}

void ParseFile::GetLine()
{
	mWorkingBuffer[0] = mToken[0] = 0;

  // tokenpos is the current position in the token
	int tokenPos = 0;

 	// get rid of whitespace (hopefully)
	while(*mPosition && (*mPosition <= ' ' && *mPosition != '\n'))
		mPosition++;

	// continue until we hit a newline
 	while(*mPosition && *mPosition != '\n')
 	{
 		// process comments
 		if(Comment());
 		// otherwise store the character and move on
 		else mWorkingBuffer[tokenPos++] = *mPosition++;
	}
	mWorkingBuffer[tokenPos] = 0;
	VarExpansion(mToken, mWorkingBuffer);
}

// get the next token from the file
char* ParseFile::NextToken()
{
	// init the token
	mWorkingBuffer[0] = mToken[0] = 0;
	
	// no file loaded, return blank
	if(!mFile)
		return mToken;
	
	// skip any leading whitespace
	while(*mPosition && *mPosition <= ' ')
		mPosition++;
	
	// we don't have any file left, return blank
	if(!*mPosition)
		return mToken;
	
	// tokenpos is the current position in the token
	int tokenPos = 0;
	
	bool inQuote = false;

	// while we are not at a token separator
	while((inQuote && *mPosition) || *mPosition > ' ')
	{
		// process comments
		if(!inQuote && Comment());
		else if(*mPosition == '"')
		{
			inQuote = !inQuote;
			mPosition++;
		}

		// otherwise store the character and move on
		else mWorkingBuffer[tokenPos++] = *mPosition++;
		
		// if we didn't store a character (comment at beginning of line)
		// and we have a valid char, cheat and skip the whitespace
		if(!tokenPos && *mPosition)
		{
			// skip whitespace
			while(*mPosition && *mPosition <= ' ')
				mPosition++;
		}
	}
	// null terminate the token
	mWorkingBuffer[tokenPos] = 0;

	// translate it
	VarExpansion(mToken, mWorkingBuffer);

	// return it
	return mToken;
}

// return the current token
char* ParseFile::GetToken()
{
	return mToken;
}	

void ParseFile::Reset()
{
	mToken[0] = mWorkingBuffer[0] = 0;
	mPosition = mFile;
}

bool ParseFile::isOk()
{
	return mFile != 0;
}

void ParseFile::Put()
{
	FILE* file = fopen(mFilename, "wb");

	if(!file)
	{
		return;
	}

	fputs(mFile,file);

	fclose(file);
}

void ParseFile::SetToken(char* newvalue)
{
	char* NewFile = new char[mFilelen+1+(strlen(newvalue) - strlen(mToken))];
	char* tmp = mFile;
	char* tmp2 = NewFile;
	
	mPosition-=strlen(mToken);
	
	while (tmp != mPosition)
		*tmp2++ = *tmp++;
	
	for (unsigned int i = strlen(newvalue); i>0; i--)
		*tmp2++ = *newvalue++;

	tmp+=strlen(mToken);
	while (tmp != (mFile+strlen(mFile)))
		*tmp2++ = *tmp++;

	*tmp2 = 0;
	delete[] mFile;
	mFile = NewFile;
}