/*
 ============================================================================
 mzscript - an advanced script module for LiteStep/PureLS
 ============================================================================

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 ============================================================================ */

#ifndef __MZSCRIPT_H
#define __MZSCRIPT_H

#include <stdio.h>
#include "lsutility.h"

#define WIN32_LEAN_AND_MEAN

//===========================================================================
// Variables
//===========================================================================

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
HINSTANCE hInstance;
HWND hwndMain, hwndParent;
int msgs[] = {LM_GETREVID, LM_REFRESH, 0};

//--------------------

typedef struct _t_varlist {
	char *name,*value,*varFile;
	bool bUpdated;
	void (*func)(char *,int);
	struct _t_varlist *next;
} t_varlist;


typedef enum {EXEC,LABEL,GOTO,GOTOIF,EXIT} LINETYPE;

typedef struct _t_execlist {
	char *exec;
	LINETYPE line;
	struct _t_execlist *next;
} t_execlist;


typedef struct _t_banglist {
	char *name;
	int refcount;
	int todelete;
	t_execlist *toexec;
	struct _t_banglist *next;
} t_banglist;

typedef struct {
	char *command;
	LINETYPE type;
} t_par;


typedef struct _t_linelist {
	char *line;
	struct _t_linelist *next;
} t_linelist;

//--------------------

char bangChar='|';
char listSep=':';
t_varlist *vars=NULL;
t_banglist *bangs=NULL;
t_linelist *rcBuffer=NULL;
int autosave=0;
unsigned int oldTokens = 0;
bool bOldSyntax;
bool bNoStartOnRefresh;
bool bRefresh = false;
bool bReadStep = false;
bool bNoCreate = false;
char returnBuffer[MAX_LINE_LENGTH];

t_par par[5] = {{"exec",EXEC},{"label",LABEL},{"goto",GOTO},
    {"gotoif",GOTOIF},{"exit",EXIT}};


//===========================================================================
// Declarations
//===========================================================================

int LoadSetup();

int varExist(char *name);
void varRemove(char *name);
void varSet(char *name,char *val,char *filename);
void varSetFunc(char *name,void (*f)(char *,int));
char *varGet(char *name);
char *getvarFile(char *name);
void varRemoveAll(void);
int varExpand(char *dest,const char *firstSrc,int n);

void addBangExec(char *bang,char *exec,LINETYPE l);
int existScriptBang(char *name);
void freeScriptBang(t_banglist *bangtmp);
void removeScriptBang(char *name);
void removeAllScriptBangs(void);

void xRes(char *place,int max);
void yRes(char *place,int max);
void bpp(char *place,int max);
void yearF(char *place,int max);
void monthF(char *place,int max);
void dayF(char *place,int max);
void weekdayF(char *place,int max);
void hourF(char *place,int max);
void minF(char *place,int max);
void secF(char *place,int max);
void mousexF(char *place,int max);
void mouseyF(char *place,int max);
//Added By Brian "Tres`ni" Hartvigsen
void milliF(char *place,int max);

void ExecCommand(HWND caller,char *szSample);

void execFirst(HWND caller,char *expr);
void execSecond(HWND caller,char *expr);

void bangSetVar(HWND caller, const char *args);
void bangAddVar(HWND caller, const char *args);
void bangMulVar(HWND caller, const char *args);
void bangIntVar(HWND caller, const char *args);
void bangShowVar(HWND caller, const char *args);
void bangMsgBox(HWND caller, const char *args);
void bangRemoveVar(HWND caller, const char *args);
void bangRunVar(HWND caller, const char *args);
void bangSaveVar(HWND caller, const char *args);
void bangReplaceVar(HWND caller, const char *args);
void bangSaveAllVars(HWND caller, const char *args);
void bangIfExist(HWND caller, const char *args);
void bangIfNExist(HWND caller, const char *args);
void bangIfEq(HWND caller, const char *args);
void bangIfNEq(HWND caller, const char *args);
int evalIf(char *e,char **theRet);
void bangIfEval(HWND caller, const char *args);
void bangScript(HWND caller, const char *bangname, const char *args);
void bangExec(HWND caller, const char *args);
void bangRemoveScript(HWND parent, const char *scriptname);
void bangSetListSep(HWND parent, const char *arg);
void pause(HWND parent, LPCTSTR pszArgs );
void bangVarDump(HWND caller, const char *args);
//Added By Brian "Tres`ni" Hartvigsen
void bangModVar(HWND caller, const char *args);
void bangRndVar(HWND caller, const char *args);

void readScriptFile(HWND parent, const char *scriptFile);
void readVarFile(char *varFile);
void readStep(HWND parent);

//===========================================================================
// External
//===========================================================================

extern "C"
{
	__declspec( dllexport ) int initModuleEx(HWND parent, HINSTANCE dllInst, LPCSTR szPath);
	__declspec( dllexport ) void quitModule(HINSTANCE dll);
}

#endif
